/*
 * Copyright (C) 2020 by Savoir-faire Linux
 * Authors: William Enright <william.enright@savoirfairelinux.com>
 *          Ndeye Anna Ndiaye <anna.ndiaye@savoirfairelinux.com>
 *          Johnny Flores <johnny.flores@savoirfairelinux.com>
 *          Mohammed Raza <mohammed.raza@savoirfairelinux.com>
 *          Felix Sidokhine <felix.sidokhine@savoirfairelinux.com>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.jami.datastore.dao;

import lombok.extern.slf4j.Slf4j;

import net.jami.datastore.main.DataStore;
import net.jami.jams.common.dao.connectivity.SQLConnection;
import net.jami.jams.common.objects.contacts.Contact;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.List;

@Slf4j
public class ContactDao extends AbstractDao<Contact> {

    public ContactDao() {
        this.setTableName("contacts");
        this.setTClass(Contact.class);
    }

    public List<Contact> getByOwner(String owner) {
        ResultSet rs = executeQuery("SELECT * FROM contacts WHERE owner = ?", owner);
        return getObjectsFromResultSet(rs);
    }

    // Not used because the strategy here is different.
    @Override
    public boolean storeObject(Contact object) {
        String query =
                "INSERT INTO contacts (owner, uri, displayName, timestamp, status)"
                        + " VALUES (?, ?, ?, ?, ?)";
        return executeInsert(query, object);
    }

    public boolean storeContactList(List<Contact> contactList) {
        SQLConnection connection = DataStore.connectionPool.getConnection();
        try {
            String delete = "DELETE FROM contacts WHERE owner = ?";
            PreparedStatement ps = connection.getConnection().prepareStatement(delete);
            ps.setString(1, contactList.get(0).getOwner());
            ps.executeUpdate();
            String insert =
                    "INSERT INTO contacts (owner, uri, displayName, timestamp, status, banned, confirmed, conversationId) VALUES "
                            + "(?, ?, ?, ?, ?, ?, ?, ?)";
            for (Contact contact : contactList) {
                ps = connection.getConnection().prepareStatement(insert);
                contact.getInsert(ps);
                ps.executeUpdate();
            }
            return true;
        } catch (Exception e) {
            log.error("Could not update contacts: {}", e.getMessage());
            return false;
        } finally {
            DataStore.connectionPool.returnConnection(connection);
        }
    }

    public boolean deleteObject(String owner, String uri) {
        String query = "DELETE FROM contacts WHERE owner = ? AND uri = ?";
        return executeUpdate(query, List.of(owner, uri));
    }
}
